/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.storage.model;

import com.google.api.client.json.GenericJson;
import com.google.api.client.json.JsonString;
import com.google.api.client.util.DateTime;
import com.google.common.primitives.UnsignedLong;

/**
 * An object.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the Cloud Storage API. For a detailed explanation see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class StorageObject extends GenericJson {

  /**
   * Access controls on the object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<ObjectAccessControl> acl;

  static {
    // hack to force ProGuard to consider ObjectAccessControl used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(ObjectAccessControl.class);
  }

  /**
   * The bucket containing this object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String bucket;

  /**
   * Cache-Control directive for the object data.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String cacheControl;

  /**
   * Number of underlying components that make up this object. Components are accumulated by compose
   * operations and are limited to a count of 32.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer componentCount;

  /**
   * Content-Disposition of the object data.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String contentDisposition;

  /**
   * Content-Encoding of the object data.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String contentEncoding;

  /**
   * Content-Language of the object data.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String contentLanguage;

  /**
   * Content-Type of the object data.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String contentType;

  /**
   * CRC32c checksum, as described in RFC 4960, Appendix B; encoded using base64.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String crc32c;

  /**
   * HTTP 1.1 Entity tag for the object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String etag;

  /**
   * The content generation of this object. Used for object versioning.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long generation;

  /**
   * The ID of the object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String id;

  /**
   * The kind of item this is. For objects, this is always storage#object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String kind;

  /**
   * MD5 hash of the data; encoded using base64.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String md5Hash;

  /**
   * Media download link.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String mediaLink;

  /**
   * User-provided metadata, in key/value pairs.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.Map<String, String> metadata;

  /**
   * The generation of the metadata for this object at this generation. Used for metadata
   * versioning. Has no meaning outside of the context of this generation.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long metageneration;

  /**
   * The name of this object. Required if not specified by URL parameter.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String name;

  /**
   * The owner of the object. This will always be the uploader of the object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Owner owner;

  /**
   * The link to this object.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String selfLink;

  /**
   * Content-Length of the data in bytes.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private UnsignedLong size;

  /**
   * Deletion time of the object in RFC 3339 format. Will be returned if and only if this version of
   * the object has been deleted.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime timeDeleted;

  /**
   * Modification time of the object metadata in RFC 3339 format.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime updated;

  /**
   * Access controls on the object.
   * The value returned may be {@code null}.
   */
  public java.util.List<ObjectAccessControl> getAcl() {
    return acl;
  }

  /**
   * Access controls on the object.
   * The value set may be {@code null}.
   */
  public StorageObject setAcl(java.util.List<ObjectAccessControl> acl) {
    this.acl = acl;
    return this;
  }

  /**
   * The bucket containing this object.
   * The value returned may be {@code null}.
   */
  public String getBucket() {
    return bucket;
  }

  /**
   * The bucket containing this object.
   * The value set may be {@code null}.
   */
  public StorageObject setBucket(String bucket) {
    this.bucket = bucket;
    return this;
  }

  /**
   * Cache-Control directive for the object data.
   * The value returned may be {@code null}.
   */
  public String getCacheControl() {
    return cacheControl;
  }

  /**
   * Cache-Control directive for the object data.
   * The value set may be {@code null}.
   */
  public StorageObject setCacheControl(String cacheControl) {
    this.cacheControl = cacheControl;
    return this;
  }

  /**
   * Number of underlying components that make up this object. Components are accumulated by compose
   * operations and are limited to a count of 32.
   * The value returned may be {@code null}.
   */
  public Integer getComponentCount() {
    return componentCount;
  }

  /**
   * Number of underlying components that make up this object. Components are accumulated by compose
   * operations and are limited to a count of 32.
   * The value set may be {@code null}.
   */
  public StorageObject setComponentCount(Integer componentCount) {
    this.componentCount = componentCount;
    return this;
  }

  /**
   * Content-Disposition of the object data.
   * The value returned may be {@code null}.
   */
  public String getContentDisposition() {
    return contentDisposition;
  }

  /**
   * Content-Disposition of the object data.
   * The value set may be {@code null}.
   */
  public StorageObject setContentDisposition(String contentDisposition) {
    this.contentDisposition = contentDisposition;
    return this;
  }

  /**
   * Content-Encoding of the object data.
   * The value returned may be {@code null}.
   */
  public String getContentEncoding() {
    return contentEncoding;
  }

  /**
   * Content-Encoding of the object data.
   * The value set may be {@code null}.
   */
  public StorageObject setContentEncoding(String contentEncoding) {
    this.contentEncoding = contentEncoding;
    return this;
  }

  /**
   * Content-Language of the object data.
   * The value returned may be {@code null}.
   */
  public String getContentLanguage() {
    return contentLanguage;
  }

  /**
   * Content-Language of the object data.
   * The value set may be {@code null}.
   */
  public StorageObject setContentLanguage(String contentLanguage) {
    this.contentLanguage = contentLanguage;
    return this;
  }

  /**
   * Content-Type of the object data.
   * The value returned may be {@code null}.
   */
  public String getContentType() {
    return contentType;
  }

  /**
   * Content-Type of the object data.
   * The value set may be {@code null}.
   */
  public StorageObject setContentType(String contentType) {
    this.contentType = contentType;
    return this;
  }

  /**
   * CRC32c checksum, as described in RFC 4960, Appendix B; encoded using base64.
   * The value returned may be {@code null}.
   */
  public String getCrc32c() {
    return crc32c;
  }

  /**
   * CRC32c checksum, as described in RFC 4960, Appendix B; encoded using base64.
   * The value set may be {@code null}.
   */
  public StorageObject setCrc32c(String crc32c) {
    this.crc32c = crc32c;
    return this;
  }

  /**
   * HTTP 1.1 Entity tag for the object.
   * The value returned may be {@code null}.
   */
  public String getEtag() {
    return etag;
  }

  /**
   * HTTP 1.1 Entity tag for the object.
   * The value set may be {@code null}.
   */
  public StorageObject setEtag(String etag) {
    this.etag = etag;
    return this;
  }

  /**
   * The content generation of this object. Used for object versioning.
   * The value returned may be {@code null}.
   */
  public Long getGeneration() {
    return generation;
  }

  /**
   * The content generation of this object. Used for object versioning.
   * The value set may be {@code null}.
   */
  public StorageObject setGeneration(Long generation) {
    this.generation = generation;
    return this;
  }

  /**
   * The ID of the object.
   * The value returned may be {@code null}.
   */
  public String getId() {
    return id;
  }

  /**
   * The ID of the object.
   * The value set may be {@code null}.
   */
  public StorageObject setId(String id) {
    this.id = id;
    return this;
  }

  /**
   * The kind of item this is. For objects, this is always storage#object.
   * The value returned may be {@code null}.
   */
  public String getKind() {
    return kind;
  }

  /**
   * The kind of item this is. For objects, this is always storage#object.
   * The value set may be {@code null}.
   */
  public StorageObject setKind(String kind) {
    this.kind = kind;
    return this;
  }

  /**
   * MD5 hash of the data; encoded using base64.
   * The value returned may be {@code null}.
   */
  public String getMd5Hash() {
    return md5Hash;
  }

  /**
   * MD5 hash of the data; encoded using base64.
   * The value set may be {@code null}.
   */
  public StorageObject setMd5Hash(String md5Hash) {
    this.md5Hash = md5Hash;
    return this;
  }

  /**
   * Media download link.
   * The value returned may be {@code null}.
   */
  public String getMediaLink() {
    return mediaLink;
  }

  /**
   * Media download link.
   * The value set may be {@code null}.
   */
  public StorageObject setMediaLink(String mediaLink) {
    this.mediaLink = mediaLink;
    return this;
  }

  /**
   * User-provided metadata, in key/value pairs.
   * The value returned may be {@code null}.
   */
  public java.util.Map<String, String> getMetadata() {
    return metadata;
  }

  /**
   * User-provided metadata, in key/value pairs.
   * The value set may be {@code null}.
   */
  public StorageObject setMetadata(java.util.Map<String, String> metadata) {
    this.metadata = metadata;
    return this;
  }

  /**
   * The generation of the metadata for this object at this generation. Used for metadata
   * versioning. Has no meaning outside of the context of this generation.
   * The value returned may be {@code null}.
   */
  public Long getMetageneration() {
    return metageneration;
  }

  /**
   * The generation of the metadata for this object at this generation. Used for metadata
   * versioning. Has no meaning outside of the context of this generation.
   * The value set may be {@code null}.
   */
  public StorageObject setMetageneration(Long metageneration) {
    this.metageneration = metageneration;
    return this;
  }

  /**
   * The name of this object. Required if not specified by URL parameter.
   * The value returned may be {@code null}.
   */
  public String getName() {
    return name;
  }

  /**
   * The name of this object. Required if not specified by URL parameter.
   * The value set may be {@code null}.
   */
  public StorageObject setName(String name) {
    this.name = name;
    return this;
  }

  /**
   * The owner of the object. This will always be the uploader of the object.
   * The value returned may be {@code null}.
   */
  public Owner getOwner() {
    return owner;
  }

  /**
   * The owner of the object. This will always be the uploader of the object.
   * The value set may be {@code null}.
   */
  public StorageObject setOwner(Owner owner) {
    this.owner = owner;
    return this;
  }

  /**
   * The link to this object.
   * The value returned may be {@code null}.
   */
  public String getSelfLink() {
    return selfLink;
  }

  /**
   * The link to this object.
   * The value set may be {@code null}.
   */
  public StorageObject setSelfLink(String selfLink) {
    this.selfLink = selfLink;
    return this;
  }

  /**
   * Content-Length of the data in bytes.
   * The value returned may be {@code null}.
   */
  public UnsignedLong getSize() {
    return size;
  }

  /**
   * Content-Length of the data in bytes.
   * The value set may be {@code null}.
   */
  public StorageObject setSize(UnsignedLong size) {
    this.size = size;
    return this;
  }

  /**
   * Deletion time of the object in RFC 3339 format. Will be returned if and only if this version of
   * the object has been deleted.
   * The value returned may be {@code null}.
   */
  public DateTime getTimeDeleted() {
    return timeDeleted;
  }

  /**
   * Deletion time of the object in RFC 3339 format. Will be returned if and only if this version of
   * the object has been deleted.
   * The value set may be {@code null}.
   */
  public StorageObject setTimeDeleted(DateTime timeDeleted) {
    this.timeDeleted = timeDeleted;
    return this;
  }

  /**
   * Modification time of the object metadata in RFC 3339 format.
   * The value returned may be {@code null}.
   */
  public DateTime getUpdated() {
    return updated;
  }

  /**
   * Modification time of the object metadata in RFC 3339 format.
   * The value set may be {@code null}.
   */
  public StorageObject setUpdated(DateTime updated) {
    this.updated = updated;
    return this;
  }

  /**
   * The owner of the object. This will always be the uploader of the object.
   */
  public static final class Owner extends GenericJson {

    /**
     * The entity, in the form user-userId.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String entity;

    /**
     * The ID for the entity.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String entityId;

    /**
     * The entity, in the form user-userId.
     * The value returned may be {@code null}.
     */
    public String getEntity() {
      return entity;
    }

    /**
     * The entity, in the form user-userId.
     * The value set may be {@code null}.
     */
    public Owner setEntity(String entity) {
      this.entity = entity;
      return this;
    }

    /**
     * The ID for the entity.
     * The value returned may be {@code null}.
     */
    public String getEntityId() {
      return entityId;
    }

    /**
     * The ID for the entity.
     * The value set may be {@code null}.
     */
    public Owner setEntityId(String entityId) {
      this.entityId = entityId;
      return this;
    }

  }

}
